#ifndef __ODBCSDK__
/* ODBCSDK.H -- Client SDK-specific ODBCAPI Definitions
 * 082801
 *
 *            (c) Copyright August Software, USA, 1994-2003.
 *           Confidential & Proprietary.  All Rights Reserved.
 *
 *  ODBCAPI connects your application to ODBC drivers installed on a
 *  centralized Windows system.   (See also http://odbcrouter.com/)
 *
 *  Why use ODBCAPI?
 *
 *    After several years of trying to get ODBC going on Macs and UNIX,
 *    there are still less than a dozen "real" ODBC drivers available on
 *    all non-Windows platforms combined.  Those ODBC drivers that are
 *    available only work with very specific versions of a certain few
 *    databases.  Therefore, it makes little sense to build applications
 *    that rely on non-Windows ODBC Driver Managers.  (Such applications
 *    must be configured by end-users using control panels and supported
 *    for the multiple versions of multiple third-party ODBC Driver
 *    Managers must be provided by your tech-support department.)  The
 *    solution of using Windows ODBC drivers on a network server solves
 *    this problem, we are in the future where everyone uses such proxy
 *    servers for Internet access anyway, and using ODBCAPI, is completely
 *    transparent to your application.
 *
 *  How does ODBCAPI SDK work?
 * 
 *    Instead of linking your application to a ODBC Driver Manager and
 *    requiring your users to interact with its ODBC Control Panel, you
 *    link it to our ODBCAPI shared-library/DLL and provide the address
 *    of the network server hosting the Windows ODBC driver.  No other
 *    code changes are required.
 *
 *  How do I provide the network server's address to ODBCAPI?
 *
 *    There are two ways: [ 1 ] Write a little function that takes a
 *    string containing a "Data Source Name" (DSN) and returns the name
 *    of the network server hosting it (from your App's existing
 *    customer preferences).  You can then call a routine in ODBCAPI
 *    (before making any other ODBC calls) to "install" your function.
 *    [ 2 ] On some platforms (LINUX, DARWIN, etc..), you or your user
 *    can setup an /etc/odbcrouter.conf file that contains this
 *    information.  With either method, you can also simply return the
 *    same network server address for all data source names if you do
 *    not want to implement a list.
 *
 *  Sometimes an ODBC driver needs to prompt the User for a username/
 *  password.  How do I handle this with the driver running on a network
 *  server?
 *
 *    You can either prompt your user (in advance) to supply this and
 *    then pass it in the ODBC connection-string (so the driver never
 *    needs to ask the user); or you can write a function to prompt
 *    the user "on demand" --and then call a routine in ODBCAPI
 *    (before making any connection attempts) that "installs" your
 *    function.
 *
 *  Which "official" ODBC API is implemented?
 *
 *    The ODBCAPI implements the standard ODBC 2.1 specification that
 *    everybody is using.  (See PDF file below.)
 *
 *  What about ODBC 3.x?
 *
 *    Most database vendors are today shipping ODBC 2.x drivers and most
 *    applications out there are written to the ODBC 2.x API.  ODBC 3 is
 *    very different than ODBC 2.x and retains full compatibility with
 *    ODBC 2, so it is only very useful if you are building a new app
 *    from scratch.  If you were to use ODBC 3.x API calls, then most of
 *    the time, the ODBC Driver Manager will be converting those calls
 *    to the ODBC 2.x driver calls, but by writing your application to
 *    the ODBC 2.x API, you avoid this costly overhead when working with
 *    most databases while still obtaining compatibility with newer
 *    ODBC 3.x drivers.  ODBC makes all of this transparent to you.
 *
 *  Which standard ODBC APIs are NOT implemented by ODBCAPI?
 *
 *    ODBCAPI does not implement the optional SQLExtendedFetch function
 *    nor the driver-management functions (which are not needed since
 *    all driver management is done on the Windows server, not on the
 *    desktop).  If we implemented SQLExtendedFetch over a network, then
 *    we would have to make "assumptions" about "how-much" and "when" your
 *    application wants data downloaded from the server into the local
 *    memory space of the user's computer.  Performance is much better
 *    when applications decide when and what to retrieve.  Since this
 *    function is not implemented in many ODBC drivers anyway, we return
 *    a 0 in SQLGetFunctions for this API to cause the app to "fall-back"
 *    to normal retrieval.  Also note that there are many free C/C++
 *    libraries available for you to use that implement cursors using
 *    the standard ODBC API.  Since ODBCAPI exports a standard ODBC API,
 *    you can use all of them.
 *
 *  Where do I obtain the ODBC API reference manual for this popular
 *  version of ODBC?
 *
 *    Extract the (PDF) file: "ODBC 2.1 Programmer's Reference Guide"
 *    in ftp://ftp.apple.com/developer/Development_Kits/MacODBC.sit.hqx
 *    This book is a direct rip of the original MS publication and it is
 *    applicable to all platforms.  CAUTION: DO NOT _INSTALL_ ANY OF THE
 *    BINARIES IN THIS .SIT FILE ON YOUR MAC.  We may provide this
 *    documentation in a future version of this SDK.
 *
 *  Reference Manual for the non-ODBC Functions in ODBCAPI
 *
 *  Callback: FindGWInfo  -- Find Gateway Information
 *  [required, MacOS-Classic, MacOS-Carbon and Win32]
 *  [optional, LINUX, DARWIN]
 *
 *  Your application may install a routine that the ODBCAPI can
 *  call to obtain the network address of the ODBC ROUTER hosting any
 *  given data source.  You must pass the address of this routine
 *  to ORSDK_InstallFindGWInfo at least one time before attempting
 *  to connect to a datasource (with SQLConnect, SQLDriverConnect
 *  or SQLBrowseConnect).  Your routine must return either SQL_SUCCESS
 *  or SQL_ERROR.
 *
 *  RETCODE FindGWInfo( dsn, gwId, hostName, dirName, disAuthPrompt,
 *                      szDefUser, szConnOpts )
 *
 *  Inputs: unsigned char *dsn    datasource being researched
 *  Outputs: uchar *gwId,         reserved, must be 1
 *           char *hostName,      "A#appletalkname" or "I#dnsname" or
 *                                          "I#x.x.x.x" (IP address)
 *           char *dirName,       null-terminated AppleTalk-zonename or
 *                                null-terminated literal "Internet"
 *           char *disAuthPrompt, 1 or 0 to enable/disable user/passwd
 *                                dialog, if 0, then PromptForAuthRtn
 *                                      must be installed
 *           char *szDefUser,     null-terminated default-username
 *                                   will be passed to PromptForAuthRtn
 *           char *szConnOpts     null-terminated optional-connopts
 *
 *  ---------------------------------------------------------------------
 *
 *  Callback:  PromptForDSNRtn - Prompt user (display dialog, etc..) to
 *                               select an ODBC ROUTER DSN to connect.
 *  [optional, all platforms]
 *
 *  Your application may optionally install a routine that the ODBCAPI
 *  can call to prompt the user to select a DSN from a list of all
 *  available DSNs maintained by your application.  You may pass the
 *  address of this routine to ORSDK_InstallPromptForDSN at least one
 *  time before attempting to connect to a NULL data source (with
 *  SQLConnect, SQLDriverConnect or SQLBrowseConnect).  The routine being
 *  installed must return either SQL_SUCCESS or SQL_ERROR.
 *
 *  RETCODE PromptForDSN( hWnd, lpszSelectedDSN, lpszConnStr )
 *
 *  Inputs: WindowPtr hWnd,         HWND/GrafPtr/WindowPtr to center-over
 *  Outputs:
 *          UCHAR *lpszSelectedDSN  buffer to receive null-terminated DSN
 *                                  selected by user
 *          UCHAR *lpszConnStr      buffer to receive null-terminated
 *                                  connection-string
 *
 *  ---------------------------------------------------------------------
 *
 *  Callback:  PromptForAuthRtn - Prompt user (display dialog, etc..) to
 *                                enter their username and password
 *  [optional, all platforms]
 *
 *  Your application may optionally install a routine that the ODBCAPI can
 *  call in order to prompt the user for a username/password associated
 *  with a particular DSN.  You may pass the address of this routine to
 *  ORSDK_InstallPromptForAuth at least one time before attempting to 
 *  connect to a datasource (with SQLConnect, SQLDriverConnect or
 *  SQLBrowseConnect) without a username or password.  The routine must
 *  return either TRUE for success or FALSE for failure.
 *
 *  BOOL PromptForAuth( hWnd, lpszDSN, lpszDefaultUser, lpszConnStrIn,
 *                      lppcbConnStrOutMax, lpszConnStrOut )
 *
 *  Inputs: WindowPtr hWnd,         HWND/GrafPtr/WindowPtr to center-over
 *          char *lpszDSN,          null-terminated DSN
 *          char *lpszDefaultUser,  null-terminated default-username
 *          char *lpszConnStrIn,    null-terminated connection-string
 *          short *lpcbConnStrOutMax sizeof output buffer lpszConnStrOut
 *  Outputs: char *lpszConnStrOut   null-terminated connection-string
 *
 *  ---------------------------------------------------------------------
 *
 *  History
 *  ======================================================================
 *  ASC 082801 Original Release
 *  ASC 090901 disAuthPrompt is now 1 or 0
 *  ASC 091001 WindowPtr def for Windows platforms
 *  ASC 121101 Added HWND, BOOL, RETCODE def for non-Windows platforms
 *  ASC 010502 WindowPtr def for Mac platforms, updated header docs
 *  ASC 012102 Added #include <sqlext.h> and ODBCVER 0x0210
 *  ASC 012302 #ifdef'd ODBCVER, added and updated comments, removed HWND
 *  ASC 060702 Support for non-VC++ compilers on Windows
 *  ASC 012803 Added lpszConnStr to GUI_PromptForDSN
 */

#ifndef ODBCVER
#define ODBCVER 0x0210
#endif
#ifdef __MACOS__
#include <macsqlext.h>
#else
#include <sqlext.h>
#endif

#ifndef BOOL
#define BOOL int
#endif
#ifndef RETCODE
#define RETCODE short
#endif

#ifndef WIN32
#define __stdcall
#define DLL
#else
#ifndef DLL
#define DLL __declspec(dllimport)
#endif
#endif

/* prototypes for ODBCSDK/ODBCAPI callbacks  */

typedef RETCODE (*FindGWInfoRtn)(unsigned char *, unsigned char *, char *,
                                 char *, char *, char *, char *);

typedef RETCODE (*PromptForDSNRtn)(void *, unsigned char *, unsigned char *);

typedef BOOL (*PromptForAuthRtn)(void *, char *, char *, char *,short *,
                                 char *);

/* non-ODBC entrypoints in ODBCSDK/ODBCAPI library */

#ifdef __cplusplus
extern "C"
{
#endif

DLL void __stdcall ORSDK_InstallFindGWInfo( FindGWInfoRtn );
DLL void __stdcall ORSDK_InstallPromptForDSN( PromptForDSNRtn );
DLL void __stdcall ORSDK_InstallPromptForAuth( PromptForAuthRtn );

#ifdef __cplusplus
}
#endif

#define __ODBCSDK__
#endif
